/*global define*/
define('de_epages/presentation/recaptcha', [
    'jquery',
    'ep'
], function ($, ep) {

    'use strict';

    var unity = (ep.config.unity!==undefined && ep.config.unity===true) ? true : undefined;

    /**
     * Creates a new object to handle Google ReCaptcha functionalities.
     *
     * ### Example
     *
     * Creates a new object to handle Google ReCaptcha functionalities.
     *
     * HTML:
     *
     *   <div id="WidgetID" class="ep-recaptcha-elem" data-methodtype="async"></div>
     *   <script src="https://www.google.com/recaptcha/api.js?onload=onloadCallback&render=explicit&hl=#lang" async defer></script>
     *
     * JavaScript:
     *    // Working example see: DE_EPAGES/Product/Templates/SF/SF.INC-EmailNotificationDialog.html
     *
     *    // Initialize require recaptcha module
     *    var recaptchaObject = recaptcha({
     *      elem: document.querySelector('#WidgetID'),
     *      sitekey: '#System.ReCaptchaPublicKey',
     *      callback: function(options) {
     *        submitEmailNotification(options);
     *      },
     *      size: 'invisible',
     *      grecaptcha: grecaptcha
     *    });
     *
     *    // ... some other code fragments ...
     *
     *    // execute the recaptcha where it is needed
     *    if (recaptchaObject) {
     *      recaptchaObject.execute()
     *    }
     *
     *    // callback submitEmailNotification
     *    function submitEmailNotification(options) {
     *      // TODO: your execution
     *    }
     *
     * @param {Object}   options                            The configuration of the object recaptcha.
     * @param {Function} [options.callback]                 A callback function which is executed immediately after the Google recaptcha request
     *                                                      was request sent.
     * @param {String} [options.sitekey]                    A valid Google recaptcha public sitekey.
     * @param {String} [options.size]                       A valid Google recaptcha option to configure the visibility.
     * @param {String} [options.elem]                       A valid DOM element which is the root of the captcha.
     * @param {String} [options.grecaptcha]                 The Google captcha reference on the window object.
     */
    return function(options) {
      // define defaults
      var defaults = {
            sitekey: '',
            callback: function(token) {},
            size: 'invisible'
          },
          captchaId;

      // leave module if elem or grecaptcha is not defined
      if (options && (options.elem === undefined || options.grecaptcha === undefined)) {
        return false;
      }

      // extends defaults with given parameters
      defaults = $.extend(defaults, options);

      // render captcha widget
      // https://developers.google.com/recaptcha/docs/display#js_api
      captchaId = defaults.grecaptcha.render(defaults.elem, {
        'sitekey' : defaults.sitekey,
        'callback' : function (token) {
          // execute given callback
          if (typeof  defaults.callback === 'function') {
            defaults.callback({
              token: token
            });
          }
        },
        'size' : defaults.size
      });

      return {
        execute: function() {
          if (captchaId !== undefined) {
            // Reset reCaptcha before using it.
            defaults.grecaptcha.reset(captchaId);
            // Execute Captcha
            defaults.grecaptcha.execute(captchaId);
          }
        }
      };
    };
});
